<?php
namespace Teknisa\PLA\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class PratoEquivalenteRelatorio extends \Zeedhi\Framework\Controller\Report {
    
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $util;
    protected $connection;
    protected $nrlista;
    protected $environment;
    
    public function __construct($reportService, $qr2Strategy, $birtStrategy, \PLA\Util\Util $util, $entityManager, $environment) {

        $this->qr2Strategy  = $qr2Strategy;
        $this->birtStrategy = $birtStrategy;
        $this->util = $util;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment; 
        parent::__construct($reportService);
    }
    
    protected $reportMapping = array (
    	'PLA90100' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'NRORG'             =>  'P_NRORG',
                'IDPADRAO'          =>  'P_IDPADRAO',
				'IDLOCAL'	        =>  'P_IDLOCAL',
                'IDPORSERVICO'      =>	'P_IDPORSERVICO',
				'CDPRATOINI'  	    =>	'P_CDPRATOINI',
				'CDPRATOFIN'        =>  'P_CDPRATOFIN',
				'NRLISTAFILIAL'     =>	'P_FILIAIS',
				'NRLISTASERVICO'    =>	'P_SERVICOS'
			)
		)
	);
	
	public function getTodosFiliais($grupoUnidade) {
	    $query = "SELECT F.CDFILIAL
          FROM FILIAL F
         WHERE EXISTS 
            (
                SELECT 1 FROM FILIOPER WHERE CDFILIAL = F.CDFILIAL AND CDOPERADOR = :CDOPERADOR)
                AND (
                    (:CDGRUPFILI = 'N') OR 
                    EXISTS(SELECT 1 FROM FILGRUPFILI FG WHERE FG.CDFILIAL = F.CDFILIAL AND FG.CDGRUPFILI = :CDGRUPFILI))";
        
        $params = [
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "CDGRUPFILI" => $grupoUnidade
        ];
                   
        $sql = $this->connection->prepare($query);
        $retorno = $this->connection->fetchAll($query, $params);
        
        $filiais = [];
        
        foreach($retorno as $filial) {
            array_push($filiais, $filial['CDFILIAL']);
        }
        
        return $filiais;
	}
	
	public function getTodosServicos($nrlistafilial) {
	    $query = "SELECT S.CDFILIAL || S.CDSERVFILI AS CHAVE
                FROM SERVICO S
                WHERE EXISTS (SELECT 1 FROM LISTA WHERE NRLISTA = :NRLISTA AND CDLISTA = S.CDFILIAL)";
                
        $params = [
            "NRLISTA" => $nrlistafilial
        ];
                    
        $sql = $this->connection->prepare($query);
        $retorno = $this->connection->fetchAll($query, $params);                    
        
        $servicos = [];
        
        foreach($retorno as $servico) {
            array_push($servicos, $servico['CHAVE']);
        }
        
        return $servicos;        
	}
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $row['NRORG'] = $this->environment->getNrOrg();
        $row['IDPADRAO'] = $row['IDPADRAO'] == 'N' ? $row['IDPADRAO'] = 'X' : $row['IDPADRAO'] = $row['IDPADRAO'];
    	$row['IDLOCAL']  = $row['IDLOCAL'] == 'N' ? $row['IDLOCAL'] = 'X' : $row['IDLOCAL'] = $row['IDLOCAL'];
    	$row['IDPORSERVICO']  = $row['IDPORSERVICO'] == 'N' ? $row['IDPORSERVICO'] = 'X' : $row['IDPORSERVICO'] = $row['IDPORSERVICO'];
        $row['CDGRUPFILI'] = !isset($row['CDGRUPFILI']) || $row['CDGRUPFILI'] == "" ? 'N' : $row['CDGRUPFILI'];
        $row['GRUPOUNIDADE'] = $row['CDGRUPFILI'];
        
        $nrlistafilial = 'LISTAVAZIA';
        if ($row['IDLOCAL'] !== 'X' || $row['IDPORSERVICO'] !== 'X'){
            $nmFilial = $row["Todos_Unidade"] == 'T' ? $this->getTodosFiliais($row['GRUPOUNIDADE']) : $row['NMFILIAL'];
            $nrlistafilial = $this->util->insertLista($nmFilial);
        }
        
        $nrlistaserv = 'LISTAVAZIA';
        
        if ($row['IDPORSERVICO'] !== 'X'){
            $nmServico = $row["Todos_Servico"] == 'T' ? $this->getTodosServicos($nrlistafilial) : $row['NMSERVICO'];
            $nrlistaserv = $this->util->insertLista($nmServico);
        }
        
        $row['NRLISTAFILIAL'] = $nrlistafilial;
        $row['NRLISTASERVICO'] = $nrlistaserv;
    }
}